#include"map.h"


//ワールドマップデータをごっそり読み込み
int WmapDataLoad(WMAP wmap[]){
	char filename[50];
	int i;
	for(i=0; i<MAP_NUM; i++){
		sprintf(filename, "data/map/map%03d.dat",i+1);
		if(WmapLoad(&wmap[i],filename) == -1){
			printf("%s読み込みミス\n",filename);
			return -1;
		}
	}
	return 0;
}
//マップデータ読み込み
int WmapLoad(WMAP *wmap, const char *filename){
	FILE *fp;
	char buf;
	int i = 0;

	//ファイルオープン
	if((fp = fopen(filename, "rb")) == NULL)return -1;

	fread(&wmap->w, sizeof(int), 1, fp);	//幅読み込み
	fread(&wmap->h, sizeof(int), 1, fp);	//高さ読み込み

	//マップ分のメモリ確保
	if((wmap->data = (unsigned char *)malloc(sizeof(unsigned char) * wmap->w * wmap->h)) == NULL)exit(0);
	//マップデータ読み込み
	while(!feof(fp)){
		fread(&buf, sizeof(char), 1, fp);
		*(wmap->data + i++) = buf;
	}
	//ファイルクローズ
	close(fp);

	return 0;
}
//ワールドマップセット（回転）
void SetWmapRotation(WMAP *wmap_t, WMAP wmap_b, unsigned char rotflg){
	int w, h;

	//メモリ確保
	if((wmap_t->data = (unsigned char *)malloc(sizeof(unsigned char) * wmap_b.w * wmap_b.h)) == NULL)exit(0);

	//回転は時計回り
	//９０°
	if(rotflg == 1){
		wmap_t->w = wmap_b.h;
		wmap_t->h = wmap_b.w;
		for(h=0; h<wmap_b.h; h++){
			for(w=0; w<wmap_b.w; w++){
				*(wmap_t->data + w*wmap_t->w + wmap_t->w-1-h) = *(wmap_b.data + w + h*wmap_b.w);
			}
		}
	}
	//１８０°
	else if(rotflg == 2){
		wmap_t->w = wmap_b.w;
		wmap_t->h = wmap_b.h;
		for(h=0; h<wmap_b.h; h++){
			for(w=0; w<wmap_b.w; w++){
				*(wmap_t->data + wmap_t->w-1-w + wmap_t->w*(wmap_t->h-1-h)) = *(wmap_b.data + w + h*wmap_b.w);
			}
		}
	}
	//２７０°
	else if(rotflg == 3){
		wmap_t->w = wmap_b.h;
		wmap_t->h = wmap_b.w;
		for(h=0; h<wmap_b.h; h++){
			for(w=0; w<wmap_b.w; w++){
				*(wmap_t->data + wmap_t->w*(wmap_t->h-1-w) + h) = *(wmap_b.data + w + h*wmap_b.w);
			}
		}
	}
	//デフォルト
	else{
		wmap_t->w = wmap_b.w;
		wmap_t->h = wmap_b.h;
		for(h=0; h<wmap_b.h; h++){
			for(w=0; w<wmap_b.w; w++){
				*(wmap_t->data + w + h*wmap_t->w) = *(wmap_b.data + w + h*wmap_b.w);
			}
		}
	}
}
//マップデータ解放
void DeleteWmap(WMAP *wmap){
	free(wmap->data);	
}
//マップデータをごっそり解放
void DeleteWmapData(WMAP wmap[]){
	int i;
	for(i=0; i<MAP_NUM; i++){
		DeleteWmap(&wmap[i]);
	}
}

//ローカルマップセット
void SetLmap(LMAP *lmap, WMAP wmap){
	int w, h;

	for(h=0; h<LMAP_H; h++){
		for(w=0; w<LMAP_W; w++){
			lmap->data[h][w] = *(wmap.data + lmap->w_x + lmap->w_y*wmap.w + w + h*wmap.w);
		}
	}

}

//ローカルマップ表示
void DrawLmap(SDL_Surface *screen, SDL_Surface *img, LMAP lmap){
	int w, h;

	for(h=0; h<LMAP_H; h++){
		for(w=0; w<LMAP_W; w++){
			DrawBmp(screen, lmap.x+w*TIP_SIZE, lmap.y+h*TIP_SIZE, TIP_SIZE, TIP_SIZE, img, (int)(lmap.data[h][w])*32, 608, 32, 32);
		}
	}

}

//ローカルマップと点のあたり判定
HITPARAM HitLmap(LMAP lmap, int x, int y){
	HITPARAM ret = {0,0,0};
	int cx, cy;

	cx = (-lmap.x+x)/TIP_SIZE;
	cy = (-lmap.y+y)/TIP_SIZE;
	if(lmap.data[cy][cx]){
		ret.x = (x - (cx * TIP_SIZE + lmap.x));
		ret.y = (y - (cy * TIP_SIZE + lmap.y));
		ret.flg = 1;
	}

	return ret;
}
//ローカルマップと線のあたり判定
HITPARAM HitLmapLine(LMAP lmap, int x1, int y1, int x2, int y2){
	HITPARAM ret = {0,0,0}, buf;
	int range, i;
	double rad;
	int cx, cy;
	range = GetRange(x1, y1, x2, y2);
	rad = GetRad(x1, y1, x2, y2);

	for(i=0; i<range; i++){
		cx = cos(rad)*i + x1;
		cy = sin(rad)*i + y1;
		buf = HitLmap(lmap, cx, cy);
		if(buf.flg){
			ret.x = cx;
			ret.y = cy;
			ret.flg = 1;
			break;
		}
	}
	return ret;
}
//ワールドマップあたり（点）
HITPARAM HitWmap(WMAP wmap, LMAP lmap, int x, int y){
	HITPARAM ret = {0,0,0};
	int cx, cy;

	cx = (-lmap.x+x)/TIP_SIZE + lmap.w_x;
	cy = (-lmap.y+y)/TIP_SIZE + lmap.w_y;
	if(*(wmap.data + cx +cy*wmap.w)){
		ret.x = (x - ((cx-lmap.w_x) * TIP_SIZE + lmap.x));
		ret.y = (y - ((cy-lmap.w_y) * TIP_SIZE + lmap.y));
		ret.flg = 1;
	}

	return ret;
}
//ワールドマップあたり（線）HITPARAM HitWma
HITPARAM HitWmapLine(WMAP wmap, LMAP lmap, int x1, int y1, int x2, int y2){
	HITPARAM ret = {0,0,0}, buf;
	int range, i;
	double rad;
	int cx, cy;
	range = GetRange(x1, y1, x2, y2);
	rad = GetRad(x1, y1, x2, y2);

	for(i=0; i<range; i++){
		cx = cos(rad)*i + x1;
		cy = sin(rad)*i + y1;
		buf = HitWmap(wmap, lmap, cx, cy);
		if(buf.flg){
			ret.x = cx;
			ret.y = cy;
			ret.flg = 1;
			break;
		}
	}
	return ret;
}
