#ifndef _MAP_H
#define _MAP_H

#include<SDL/SDL.h>
#include<stdio.h>
#include<stdlib.h>
#include"support.h"

#define MAP_NUM	10	//マップ総数

#define LMAP_W	13	//ローカルマップの幅
#define LMAP_H	11	//　　　〃　　　　高さ
#define LMAP_DEF_X	-48	//ローカルマップのデフォルト表示位置 x座標
#define LMAP_DEF_Y	-56	//ローカルマップのデフォルト表示位置 y座標
#define TIP_SIZE	32	//マップチップのサイズ

//ワールドマップ
typedef struct WMAP{
	int w;	//幅
	int h;	//高さ
	unsigned char *data;	//マップデータ
}WMAP;

//ローカルマップ
typedef struct LMAP{
	int x;	//画面描画位置 x座標
	int y;	//　　　〃　　 y座標
	int w_x;	//ワールドマップ参照位置 x座標
	int w_y;	//　　　　　〃　　　　　 y座標
	unsigned char data[LMAP_H][LMAP_W];	//マップデータ
}LMAP;

//マップとのあたり判定時の戻り値
typedef struct HITPARAM{
	int x;
	int y;
	unsigned char flg;
}HITPARAM;

int WmapDataLoad(WMAP wmap[]);		//ワールドマップデータをごっそり読み込み
int WmapLoad(WMAP *wmap, const char *filename);		//ワールドマップデータ読み込み
void SetWmapRotation(WMAP *wmap_t, WMAP wmap_b, unsigned char rotflg);	//ワールドマップセット（回転）
void DeleteWmap(WMAP *wmap);					//マップデータ解放
void DeleteWmapData(WMAP wmap[]);			//マップデータをごっそり解放

void SetLmap(LMAP *lmap, WMAP wmap);			//ローカルマップセット
void DrawLmap(SDL_Surface *screen, SDL_Surface *img, LMAP lmap);		//ローカルマップ表示
HITPARAM HitLmap(LMAP lmap, int x, int y);					//ローカルマップあたり（点）
HITPARAM HitLmapLine(LMAP lmap, int x1, int y1, int x2, int y2);	//ローカルマップあたり（線）
HITPARAM HitWmap(WMAP wmap, LMAP lmap, int x, int y);					//ワールドマップあたり（点）
HITPARAM HitWmapLine(WMAP wmap, LMAP lmap, int x1, int y1, int x2, int y2);	//ワールドマップあたり（線）


#endif
